@@ -5,11 +5,14 @@
 #include "hw/pvr/spg.h"
 #include "audio/audiostream.h"
 #include "oslib/oslib.h"
+#include "oslib/storage.h"
 #include "hw/aica/sgc_if.h"
 #include "cfg/option.h"
 #include <zlib.h>
 #include <cerrno>
 #include <ctime>
+#include <thread>
+#include <chrono>
 
 const char* maple_sega_controller_name = "Dreamcast Controller";
 const char* maple_sega_vmu_name        = "Visual Memory";
@@ -31,15 +34,19 @@ const char* maple_densha_controller_name    = "TAITO 001 Controller";
 const char* maple_sega_brand = "Produced By or Under License From SEGA ENTERPRISES,LTD.";
 
 //fill in the info
-void maple_device::Setup(u32 port, int playerNum)
+void maple_device::Setup(u32 bus, u32 port, int playerNum)
 {
-	maple_port = port;
-	bus_port = maple_GetPort(port);
-	bus_id = maple_GetBusId(port);
+	maple_port = (bus << 6) | (1 << port);
+	bus_port = port;
+	bus_id = bus;
 	logical_port[0] = 'A' + bus_id;
 	logical_port[1] = bus_port == 5 ? 'x' : '1' + bus_port;
 	logical_port[2] = 0;
 	player_num = playerNum == -1 ? bus_id : playerNum;
+
+	config = new MapleConfigMap(this);
+	OnSetup();
+	MapleDevices[bus][port] = shared_from_this();
 }
 maple_device::~maple_device()
 {
@@ -331,10 +338,11 @@ u8 vmu_default[] = {
 
 struct maple_sega_vmu: maple_base
 {
-	FILE* file;
+	FILE *file = nullptr;
 	u8 flash_data[128_KB];
 	u8 lcd_data[192];
 	u8 lcd_data_decoded[48*32];
+	bool fullSaveNeeded = false;
 
 	MapleDeviceType get_device_type() override
 	{
@@ -360,11 +368,28 @@ struct maple_sega_vmu: maple_base
 				config->SetImage(lcd_data_decoded);
 				break;
 			}
+		fullSaveNeeded = true;
+	}
+
+	virtual bool fullSave()
+	{
+		if (file == nullptr)
+			return false;
+		if (std::fseek(file, 0, SEEK_SET) != 0) {
+			ERROR_LOG(MAPLE, "VMU %s: I/O error", logical_port);
+			return false;
+		}
+		if (std::fwrite(flash_data, sizeof(flash_data), 1, file) != 1) {
+			ERROR_LOG(MAPLE, "Failed to write the VMU %s to disk", logical_port);
+			return false;
+		}
+		fullSaveNeeded = false;
+		return true;
 	}
 
 	void initializeVmu()
 	{
-		INFO_LOG(MAPLE, "Initialising empty VMU...");
+		INFO_LOG(MAPLE, "Initialising empty VMU %s...", logical_port);
 
 		uLongf dec_sz = sizeof(flash_data);
 		int rv = uncompress(flash_data, &dec_sz, vmu_default, sizeof(vmu_default));
@@ -372,34 +397,44 @@ struct maple_sega_vmu: maple_base
 		verify(rv == Z_OK);
 		verify(dec_sz == sizeof(flash_data));
 
-		if (file != nullptr)
-		{
-			if (std::fwrite(flash_data, sizeof(flash_data), 1, file) != 1)
-				WARN_LOG(MAPLE, "Failed to write the VMU to disk");
-			if (std::fseek(file, 0, SEEK_SET) != 0)
-				WARN_LOG(MAPLE, "VMU: I/O error");
-		}
+		fullSave();
 	}
 
 	void OnSetup() override
 	{
 		memset(flash_data, 0, sizeof(flash_data));
 		memset(lcd_data, 0, sizeof(lcd_data));
-		std::string apath = hostfs::getVmuPath(logical_port);
 
-		file = nowide::fopen(apath.c_str(), "rb+");
-		if (file == nullptr)
-		{
-			INFO_LOG(MAPLE, "Unable to open VMU save file \"%s\", creating new file", apath.c_str());
-			file = nowide::fopen(apath.c_str(), "wb+");
-			if (file == nullptr)
-				ERROR_LOG(MAPLE, "Failed to create VMU save file \"%s\"", apath.c_str());
-			initializeVmu();
-		}
-
-		if (file != nullptr)
-			if (std::fread(flash_data, sizeof(flash_data), 1, file) != 1)
-				WARN_LOG(MAPLE, "Failed to read the VMU from disk");
+        // Load existing vmu file if found
+        std::string rpath = hostfs::getVmuPath(logical_port, false);
+		// this might be a storage url
+		FILE *rfile = hostfs::storage().openFile(rpath, "rb");
+        if (rfile == nullptr) {
+            INFO_LOG(MAPLE, "Unable to open VMU file \"%s\", creating new file", rpath.c_str());
+        }
+        else
+        {
+            if (std::fread(flash_data, sizeof(flash_data), 1, rfile) != 1)
+                WARN_LOG(MAPLE, "Failed to read the VMU file \"%s\" from disk", rpath.c_str());
+            std::fclose(rfile);
+        }
+        // Open or create the vmu file to save to
+        std::string wpath = hostfs::getVmuPath(logical_port, true);
+        file = nowide::fopen(wpath.c_str(), "rb+");
+        if (file == nullptr)
+        {
+            file = nowide::fopen(wpath.c_str(), "wb+");
+			if (file == nullptr) {
+                ERROR_LOG(MAPLE, "Failed to create VMU save file \"%s\"", wpath.c_str());
+			}
+			else if (rfile != nullptr)
+			{
+				// VMU file is being renamed so save it fully now
+				// and delete the old file
+				if (fullSave())
+					nowide::remove(rpath.c_str());
+			}
+        }
 
 		u8 sum = 0;
 		for (u32 i = 0; i < sizeof(flash_data); i++)
@@ -408,6 +443,7 @@ struct maple_sega_vmu: maple_base
 		if (sum == 0)
 			// This means the existing VMU file is completely empty and needs to be recreated
 			initializeVmu();
+		fullSaveNeeded = false;
 	}
 
 	~maple_sega_vmu() override
@@ -633,17 +669,19 @@ struct maple_sega_vmu: maple_base
 
 						if (file != nullptr)
 						{
-							if (std::fseek(file, write_adr, SEEK_SET) != 0
+							if (fullSaveNeeded) {
+								if (!fullSave())
+									return MDRE_FileError;
+							}
+							else if (std::fseek(file, write_adr, SEEK_SET) != 0
 									|| std::fwrite(&flash_data[write_adr], write_len, 1, file) != 1)
 							{
-								WARN_LOG(MAPLE, "Failed to save VMU %s: I/O error", logical_port);
+								ERROR_LOG(MAPLE, "Failed to save VMU %s: I/O error", logical_port);
 								return MDRE_FileError; // I/O error
 							}
-							std::fflush(file);
 						}
-						else
-						{
-							INFO_LOG(MAPLE, "Failed to save VMU %s data", logical_port);
+						else {
+							WARN_LOG(MAPLE, "Failed to save VMU %s data", logical_port);
 						}
 						return MDRS_DeviceReply;
 					}
@@ -651,7 +689,7 @@ struct maple_sega_vmu: maple_base
 					case MFID_2_LCD:
 					{
 						DEBUG_LOG(MAPLE, "VMU %s LCD write", logical_port);
-						r32();
+						r32();	// PT, phase, block#
 						rptr(lcd_data,192);
 
 						u8 white=0xff,black=0x00;
@@ -978,7 +1016,15 @@ struct maple_sega_purupuru : maple_base
 			//2
 			w16(0x0640);	// 160 mA
 
-			return cmd == MDC_DeviceRequest ? MDRS_DeviceStatus : MDRS_DeviceStatusAll;
+			if (cmd == MDC_AllStatusReq)
+			{
+				const char *extra = "Version 1.000,1998/11/10,315-6211-AH   ,Vibration Motor:1,Fm:4 - 30Hz,Pow:7     ";
+				wptr(extra, strlen(extra));
+				return MDRS_DeviceStatusAll;
+			}
+			else {
+				return MDRS_DeviceStatus;
+			}
 
 			//get last vibration
 		case MDCF_GetCondition:
@@ -1598,6 +1644,51 @@ struct maple_densha_controller: maple_sega_controller
 	}
 };
 
+struct FullController : maple_sega_controller
+{
+	u32 get_capabilities() override
+	{
+		// byte 0: 0  0  0  0  0  0  0  0
+		// byte 1: 0  0  a5 a4 a3 a2 a1 a0
+		// byte 2: R2 L2 D2 U2 D  X  Y  Z
+		// byte 3: R  L  D  U  St A  B  C
+		return 0xffff3f00;	// 6 axes, all buttons
+	}
+
+	u16 getButtonState(const PlainJoystickState &pjs) override
+	{
+		u32 kcode = pjs.kcode;
+		mutualExclusion(kcode, DC_DPAD_UP | DC_DPAD_DOWN);
+		mutualExclusion(kcode, DC_DPAD_LEFT | DC_DPAD_RIGHT);
+		mutualExclusion(kcode, DC_DPAD2_UP | DC_DPAD2_DOWN);
+		mutualExclusion(kcode, DC_DPAD2_LEFT | DC_DPAD2_RIGHT);
+		return kcode;
+	}
+
+	u32 getAnalogAxis(int index, const PlainJoystickState &pjs) override
+	{
+		if (index == 4 || index == 5)
+		{
+			// Limit the magnitude of the analog axes to 128
+			s8 xaxis = pjs.joy[PJAI_X2] - 128;
+			s8 yaxis = pjs.joy[PJAI_Y2] - 128;
+			limit_joystick_magnitude<128>(xaxis, yaxis);
+			if (index == 4)
+				return xaxis + 128;
+			else
+				return yaxis + 128;
+		}
+		return maple_sega_controller::getAnalogAxis(index, pjs);
+	}
+
+	const char *get_device_name() override {
+		return "Dreamcast Controller XL";
+	}
+
+	MapleDeviceType get_device_type() override {
+		return MDT_SegaControllerXL;
+	}
+};
 
 // Emulates a 838-14245-92 maple to RS232 converter
 // wired to a 838-14243 RFID reader/writer (apparently Saxa HW210)
@@ -1634,7 +1725,7 @@ struct RFIDReaderWriter : maple_base
 		u32 resp = Dma(command, &buffer_in[1], buffer_in_len - 4, &buffer_out[1], outlen);
 
 		if (reci & 0x20)
-			reci |= maple_GetAttachedDevices(maple_GetBusId(reci));
+			reci |= maple_GetAttachedDevices(bus_id);
 
 		verify(u8(outlen / 4) * 4 == outlen);
 		buffer_out[0] = (resp << 0 ) | (reci << 8) | (send << 16) | ((outlen / 4) << 24);
@@ -1958,107 +2049,607 @@ struct RFIDReaderWriter : maple_base
 
 void insertRfidCard(int playerNum)
 {
-	maple_device *mapleDev = MapleDevices[1 + playerNum][5];
+	std::shared_ptr<maple_device> mapleDev = MapleDevices[1 + playerNum][5];
 	if (mapleDev != nullptr && mapleDev->get_device_type() == MDT_RFIDReaderWriter)
-		((RFIDReaderWriter *)mapleDev)->insertCard();
+		std::static_pointer_cast<RFIDReaderWriter>(mapleDev)->insertCard();
 }
 
 void setRfidCardData(int playerNum, u8 *data)
 {
-	maple_device *mapleDev = MapleDevices[1 + playerNum][5];
+	std::shared_ptr<maple_device> mapleDev = MapleDevices[1 + playerNum][5];
 	if (mapleDev != nullptr && mapleDev->get_device_type() == MDT_RFIDReaderWriter)
-		((RFIDReaderWriter *)mapleDev)->setCardData(data);
+		std::static_pointer_cast<RFIDReaderWriter>(mapleDev)->setCardData(data);
 }
 
 const u8 *getRfidCardData(int playerNum)
 {
-	maple_device *mapleDev = MapleDevices[1 + playerNum][5];
+	std::shared_ptr<maple_device> mapleDev = MapleDevices[1 + playerNum][5];
 	if (mapleDev != nullptr && mapleDev->get_device_type() == MDT_RFIDReaderWriter)
-		return ((RFIDReaderWriter *)mapleDev)->getCardData();
+		return std::static_pointer_cast<RFIDReaderWriter>(mapleDev)->getCardData();
 	else
 		return nullptr;
 }
 
-maple_device* maple_Create(MapleDeviceType type)
+std::shared_ptr<maple_device> maple_Create(MapleDeviceType type)
 {
-	maple_device* rv=0;
 	switch(type)
 	{
 	case MDT_SegaController:
 		if (!settings.platform.isAtomiswave())
-			rv = new maple_sega_controller();
+			return std::make_shared<maple_sega_controller>();
 		else
-			rv = new maple_atomiswave_controller();
-		break;
-
-	case MDT_Microphone:
-		rv=new maple_microphone();
-		break;
-
-	case MDT_SegaVMU:
-		rv = new maple_sega_vmu();
-		break;
-
-	case MDT_PurupuruPack:
-		rv = new maple_sega_purupuru();
-		break;
-
-	case MDT_Keyboard:
-		rv = new maple_keyboard();
-		break;
-
-	case MDT_Mouse:
-		rv = new maple_mouse();
-		break;
-
+			return std::make_shared<maple_atomiswave_controller>();
+	case MDT_Microphone:		return std::make_shared<maple_microphone>();
+	case MDT_SegaVMU:			return std::make_shared<maple_sega_vmu>();
+	case MDT_PurupuruPack:		return std::make_shared<maple_sega_purupuru>();
+	case MDT_Keyboard:			return std::make_shared<maple_keyboard>();
+	case MDT_Mouse:				return std::make_shared<maple_mouse>();
 	case MDT_LightGun:
 		if (!settings.platform.isAtomiswave())
-			rv = new maple_lightgun();
+			return std::make_shared<maple_lightgun>();
 		else
-			rv = new atomiswave_lightgun();
-		break;
-
-	case MDT_NaomiJamma:
-		rv = new maple_naomi_jamma();
-		break;
-
-	case MDT_TwinStick:
-		rv = new maple_sega_twinstick();
-		break;
-
-	case MDT_AsciiStick:
-		rv = new maple_ascii_stick();
-		break;
-
-	case MDT_MaracasController:
-		rv = new maple_maracas_controller();
-		break;
-
-	case MDT_FishingController:
-		rv = new maple_fishing_controller();
-		break;
-
-	case MDT_PopnMusicController:
-		rv = new maple_popnmusic_controller();
-		break;
-
-	case MDT_RacingController:
-		rv = new maple_racing_controller();
-		break;
-
-	case MDT_DenshaDeGoController:
-		rv = new maple_densha_controller();
-		break;
-
-	case MDT_RFIDReaderWriter:
-		rv = new RFIDReaderWriter();
-		break;
+			return std::make_shared<atomiswave_lightgun>();
+	case MDT_NaomiJamma:		return std::make_shared<maple_naomi_jamma>();
+	case MDT_TwinStick:			return std::make_shared<maple_sega_twinstick>();
+	case MDT_AsciiStick:		return std::make_shared<maple_ascii_stick>();
+	case MDT_MaracasController:	return std::make_shared<maple_maracas_controller>();
+	case MDT_FishingController:	return std::make_shared<maple_fishing_controller>();
+	case MDT_PopnMusicController:	return std::make_shared<maple_popnmusic_controller>();
+	case MDT_RacingController:	return std::make_shared<maple_racing_controller>();
+	case MDT_DenshaDeGoController:	return std::make_shared<maple_densha_controller>();
+	case MDT_SegaControllerXL:	return std::make_shared<FullController>();
+	case MDT_RFIDReaderWriter:	return std::make_shared<RFIDReaderWriter>();
 
 	default:
 		ERROR_LOG(MAPLE, "Invalid device type %d", type);
 		die("Invalid maple device type");
 		break;
 	}
-
-	return rv;
+	return nullptr;
 }
+
+#if (defined(_WIN32) || defined(__linux__) || (defined(__APPLE__) && defined(TARGET_OS_MAC))) && !defined(TARGET_UWP) && defined(USE_SDL) && !defined(LIBRETRO)
+#include "sdl/dreamlink.h"
+#include <list>
+#include <memory>
+
+struct DreamLinkVmu : public maple_sega_vmu
+{
+	bool running = true;
+
+	std::shared_ptr<DreamLink> dreamlink;
+	bool useRealVmuMemory;  //!< Set this to true to use physical VMU memory, false for virtual memory
+	std::chrono::time_point<std::chrono::system_clock> lastWriteTime;
+	bool mirroredBlocks[256]; //!< Set to true for block that has been loaded/written
+	s16 lastWriteBlock = -1;
+
+	std::list<u8> blocksToWrite;
+	std::mutex writeMutex;
+	std::condition_variable writeCv;
+	std::thread writeThread;
+
+	static u64 lastNotifyTime;
+	static u64 lastErrorNotifyTime;
+
+	DreamLinkVmu(std::shared_ptr<DreamLink> dreamlink) :
+		dreamlink(dreamlink),
+		writeThread([this](){writeEntrypoint();})
+	{
+		// Initialize useRealVmuMemory with our config setting
+		useRealVmuMemory = config::UsePhysicalVmuMemory;
+	}
+
+	virtual ~DreamLinkVmu() {
+		running = false;
+
+		// Entering lock context
+		{
+			std::unique_lock<std::mutex> lock(writeMutex);
+			writeCv.notify_all();
+		}
+
+		writeThread.join();
+	}
+
+	void OnSetup() override
+	{
+		// Update useRealVmuMemory in case config changed
+		useRealVmuMemory = config::UsePhysicalVmuMemory;
+
+		// All data must be re-read
+		memset(mirroredBlocks, 0, sizeof(mirroredBlocks));
+
+		if (useRealVmuMemory)
+		{
+			// Ensure file is not being used
+			if (file != nullptr)
+			{
+				std::fclose(file);
+				file = nullptr;
+			}
+
+			memset(flash_data, 0, sizeof(flash_data));
+			memset(lcd_data, 0, sizeof(lcd_data));
+
+		}
+		else
+		{
+			maple_sega_vmu::OnSetup();
+		}
+	}
+
+	bool fullSave() override
+	{
+		if (useRealVmuMemory)
+		{
+			// Skip virtual save when using physical VMU
+			//DEBUG_LOG(MAPLE, "Not saving because this is a real vmu");
+			NOTICE_LOG(MAPLE, "Saving to physical VMU");
+
+			return true;
+		}
+		else
+		{
+			return maple_sega_vmu::fullSave();
+		}
+	}
+
+	u32 dma(u32 cmd) override
+	{
+		// Physical VMU logic
+		if (dma_count_in >= 4)
+		{
+			const u32 functionId = *(u32*)dma_buffer_in;
+			const MapleMsg* msg = reinterpret_cast<const MapleMsg*>(dma_buffer_in - 4);
+
+			if (functionId == MFID_1_Storage)
+			{
+				if (useRealVmuMemory)
+				{
+					const u64 currentTime = std::chrono::duration_cast<std::chrono::milliseconds>(
+						std::chrono::steady_clock::now().time_since_epoch()).count();
+
+					// Only show notification once every 6 seconds to avoid spam
+					if (cmd == MDCF_BlockRead &&
+						(currentTime - lastNotifyTime) > 4000 &&
+						(currentTime - lastErrorNotifyTime) > 4000)
+					{
+						// This is a read operation (loading)
+						os_notify("ATTENTION: Loading from a physical VMU", 6000,
+								"Game data is being loaded from your physical VMU");
+						lastNotifyTime = currentTime;
+					}
+
+					switch (cmd)
+					{
+					case MDCF_BlockWrite:
+					{
+						// Throw away function
+						r32();
+
+						// Save the write to RAM
+						u32 bph=r32();
+						u32 Block = lastWriteBlock = (SWAP32(bph))&0xffff;
+						u32 Phase = ((SWAP32(bph))>>16)&0xff;
+						u32 write_adr=Block*512+Phase*(512/4);
+						u32 write_len=r_count();
+						DEBUG_LOG(MAPLE, "VMU %s block write: Block %d Phase %d addr %x len %d", logical_port, Block, Phase, write_adr, write_len);
+						if (write_adr + write_len > sizeof(flash_data))
+						{
+							INFO_LOG(MAPLE, "Failed to write VMU %s: overflow", logical_port);
+							skip(write_len);
+							return MDRE_FileError; //invalid params
+						}
+						rptr(&flash_data[write_adr],write_len);
+
+						// All done - wait until GetLastError to queue the write
+						return MDRS_DeviceReply;
+					}
+
+					case MDCF_GetLastError:
+					{
+						mirroredBlocks[lastWriteBlock] = true;
+
+						// Entering lock context
+						{
+							std::unique_lock<std::mutex> lock(writeMutex);
+							if (std::find(blocksToWrite.begin(), blocksToWrite.end(), lastWriteBlock) == blocksToWrite.end())
+							{
+								blocksToWrite.push_back(lastWriteBlock);
+								writeCv.notify_all();
+							}
+						}
+
+						lastWriteTime = std::chrono::system_clock::now();
+						return MDRS_DeviceReply;
+					}
+
+					case MDCF_BlockRead:
+					{
+						u8 requestBlock = msg->data[7];
+						if (!mirroredBlocks[requestBlock]) {
+							// Try up to 4 times to read
+							bool readSuccess = false;
+							for (u32 i = 0; i < 4; ++i) {
+								if (i > 0) {
+									std::this_thread::sleep_for(std::chrono::milliseconds(30));
+								}
+
+								MapleMsg rcvMsg;
+								if (dreamlink->send(*msg, rcvMsg) && rcvMsg.size == 130) {
+									// Something read!
+									u8 block = rcvMsg.data[7];
+									memcpy(&flash_data[block * 4 * 128], &rcvMsg.data[8], 4 * 128);
+									mirroredBlocks[block] = true;
+									readSuccess = true;
+									break;
+								}
+							}
+
+							if (!readSuccess) {
+								ERROR_LOG(MAPLE, "Failed to read VMU %s: I/O error", logical_port);
+								return MDRE_FileError; // I/O error
+							}
+						}
+
+						break;
+					}
+
+					default:
+						// do nothing
+						break;
+					}
+				}
+			}
+			else if (functionId == MFID_2_LCD)
+			{
+				if (cmd == MDCF_BlockWrite)
+				{
+					dreamlink->send(*msg);
+				}
+			}
+			else if (functionId == MFID_3_Clock)
+			{
+				if (cmd == MDCF_SetCondition)
+				{
+					dreamlink->send(*msg);
+				}
+			}
+		}
+
+		// If made it here, call base's dma to handle return value
+		return maple_sega_vmu::dma(cmd);
+	}
+
+	void copyIn(std::shared_ptr<maple_sega_vmu> other)
+	{
+		memcpy(flash_data, other->flash_data, sizeof(flash_data));
+		memcpy(lcd_data, other->lcd_data, sizeof(lcd_data));
+		memcpy(lcd_data_decoded, other->lcd_data_decoded, sizeof(lcd_data_decoded));
+		fullSaveNeeded = other->fullSaveNeeded;
+	}
+
+	void copyOut(std::shared_ptr<maple_sega_vmu> other)
+	{
+		// Never copy data to virtual VMU if physical VMU is enabled
+		if (!config::UsePhysicalVmuMemory && !useRealVmuMemory)
+		{
+			memcpy(other->flash_data, flash_data, sizeof(other->flash_data));
+			memcpy(other->lcd_data, lcd_data, sizeof(other->lcd_data));
+			memcpy(other->lcd_data_decoded, lcd_data_decoded, sizeof(other->lcd_data_decoded));
+			other->fullSaveNeeded = fullSaveNeeded;
+		}
+	}
+
+	void updateScreen()
+	{
+		MapleMsg msg;
+		msg.command = MDCF_BlockWrite;
+		msg.destAP = maple_port;
+		msg.originAP = bus_id << 6;
+		msg.size = 2 + sizeof(lcd_data) / 4;
+		*(u32*)&msg.data[0] = MFID_2_LCD;
+		*(u32*)&msg.data[4] = 0;    // PT, phase, block#
+		memcpy(&msg.data[8], lcd_data, sizeof(lcd_data));
+		dreamlink->send(msg);
+	}
+
+private:
+	//! Thread entrypoint for write operations
+	void writeEntrypoint()
+	{
+		while (true)
+		{
+			u8 block = 0;
+
+			// Entering lock context
+			{
+				std::unique_lock<std::mutex> lock(writeMutex);
+				writeCv.wait(lock, [this](){ return (!running || !blocksToWrite.empty()); });
+
+				if (!running)
+				{
+					break;
+				}
+
+				block = blocksToWrite.front();
+				blocksToWrite.pop_front();
+			}
+
+			const u64 currentTime = std::chrono::duration_cast<std::chrono::milliseconds>(
+				std::chrono::steady_clock::now().time_since_epoch()).count();
+
+			// Only show notification once every 6 seconds to avoid spam
+			if ((currentTime - lastNotifyTime) > 4000 && (currentTime - lastErrorNotifyTime) > 4000)
+			{
+				// This is a write operation (saving)
+				os_notify("ATTENTION: You are saving to a physical VMU", 6000,
+						"Do not disconnect the VMU or close the game");
+				lastNotifyTime = currentTime;
+			}
+
+			bool writeSuccess = true;
+			const u8* blockData = &flash_data[block * 4 * 128];
+			std::chrono::milliseconds delay(10);
+			const std::chrono::milliseconds delayInc(5);
+			// Try up to 4 times to write
+			for (u32 i = 0; i < 4; ++i) {
+				if (i > 0) {
+					// Slow down writes to avoid overloading the VMU
+					delay += delayInc;
+				}
+
+				writeSuccess = true;
+
+				// 4 write phases per block
+				for (u32 phase = 0; phase < 4; ++phase) {
+					MapleMsg writeMsg;
+					writeMsg.command = MDCF_BlockWrite;
+					writeMsg.destAP = (bus_id << 6) | (1 << bus_port);
+					writeMsg.originAP = (bus_id << 6);
+					writeMsg.size = 34;
+					writeMsg.setWord(MFID_1_Storage, 0);
+					const u32 locationWord = (block << 24) | (phase << 8);
+					writeMsg.setWord(locationWord, 1);
+					memcpy(&writeMsg.data[8], &blockData[phase * 128], 128);
+
+					// Delay before writing
+					std::this_thread::sleep_for(delay);
+
+					MapleMsg rcvMsg;
+					if (!dreamlink->send(writeMsg, rcvMsg) || rcvMsg.command != MDRS_DeviceReply) {
+						// Not acknowledged
+						writeSuccess = false;
+						break;
+					}
+				}
+
+				if (writeSuccess) {
+					// Delay before committing
+					std::this_thread::sleep_for(delay);
+
+					// Send the GetLastError command to commit the data
+					MapleMsg writeMsg;
+					writeMsg.command = MDCF_GetLastError;
+					writeMsg.destAP = (bus_id << 6) | (1 << bus_port);
+					writeMsg.originAP = (bus_id << 6);
+					writeMsg.size = 2;
+					writeMsg.setWord(MFID_1_Storage, 0);
+					const u32 phase = 4;
+					const u32 locationWord = (block << 24) | (phase << 8);
+					writeMsg.setWord(locationWord, 1);
+					MapleMsg rcvMsg;
+					if (dreamlink->send(writeMsg, rcvMsg) && rcvMsg.command == MDRS_DeviceReply) {
+						// Acknowledged
+						break;
+					}
+				}
+				// else: continue to retry
+			}
+
+			if (!writeSuccess) {
+				ERROR_LOG(MAPLE, "Failed to save VMU %s: I/O error", logical_port);
+
+				const u64 currentTime = std::chrono::duration_cast<std::chrono::milliseconds>(
+					std::chrono::steady_clock::now().time_since_epoch()).count();
+
+				if ((currentTime - lastErrorNotifyTime) > 4000)
+				{
+					os_notify("ATTENTION: Write to VMU failed", 6000);
+					lastErrorNotifyTime = currentTime;
+				}
+			}
+		}
+	}
+};
+
+u64 DreamLinkVmu::lastNotifyTime = 0;
+u64 DreamLinkVmu::lastErrorNotifyTime = 0;
+
+struct DreamLinkPurupuru : public maple_sega_purupuru
+{
+	std::shared_ptr<DreamLink> dreamlink;
+
+	//! Number of consecutive stop conditions sent
+	u32 stopSendCount = 0;
+
+	DreamLinkPurupuru(std::shared_ptr<DreamLink> dreamlink) : dreamlink(dreamlink) {
+	}
+
+	u32 dma(u32 cmd) override
+	{
+		if (cmd == MDCF_BlockWrite || cmd == MDCF_SetCondition) {
+			const MapleMsg *msg = reinterpret_cast<const MapleMsg*>(dma_buffer_in - 4);
+			const u32 functionId = *(u32*)dma_buffer_in;
+			const u32 condition = *(u32*)(dma_buffer_in + 4);
+			if (functionId == MFID_8_Vibration && condition == 0x00000010) {
+				++stopSendCount;
+			} else {
+				stopSendCount = 0;
+			}
+
+			// Only send 2 consecutive stop commands; ignore the rest to avoid unnecessary communications
+			if (stopSendCount <= 2) {
+				dreamlink->send(*msg);
+			}
+		}
+		return maple_sega_purupuru::dma(cmd);
+	}
+};
+
+static std::list<std::shared_ptr<DreamLinkVmu>> dreamLinkVmus[2];
+static std::list<std::shared_ptr<DreamLinkPurupuru>> dreamLinkPurupurus;
+
+static void disablePhysicalVmuMemoryOption()
+{
+	// Make setting read only
+	config::UsePhysicalVmuMemory.override(config::UsePhysicalVmuMemory);
+}
+
+static void enablePhysicalVmuMemoryOption()
+{
+	// Remove read-only setting and preserve current value
+	bool val = config::UsePhysicalVmuMemory;
+	config::UsePhysicalVmuMemory.reset();
+	config::UsePhysicalVmuMemory.set(val);
+}
+
+void createDreamLinkDevices(std::shared_ptr<DreamLink> dreamlink, bool gameStart, bool gameEnd)
+{
+	const int bus = dreamlink->getBus();
+
+	for (int i = 0; i < 2; ++i)
+	{
+		std::shared_ptr<maple_device> dev = MapleDevices[bus][i];
+
+		if ((dreamlink->getFunctionCode(i + 1) & MFID_1_Storage) || (dev != nullptr && dev->get_device_type() == MDT_SegaVMU))
+		{
+			bool vmuFound = false;
+			std::shared_ptr<DreamLinkVmu> vmu;
+			for (const std::shared_ptr<DreamLinkVmu>& vmuIter : dreamLinkVmus[i])
+			{
+				if (vmuIter->dreamlink.get() == dreamlink.get())
+				{
+					vmuFound = true;
+					vmu = vmuIter;
+					break;
+				}
+			}
+
+			if (gameStart || !vmuFound)
+			{
+				if (!vmu)
+				{
+					vmu = std::make_shared<DreamLinkVmu>(dreamlink);
+				}
+
+				vmu->Setup(bus, i);
+
+				if (!vmuFound && dev && dev->get_device_type() == MDT_SegaVMU && !vmu->useRealVmuMemory) {
+					// Only copy data from virtual VMU if Physical VMU Only is disabled
+					vmu->copyIn(std::static_pointer_cast<maple_sega_vmu>(dev));
+					if (!gameStart) {
+						vmu->updateScreen();
+					}
+				}
+
+				if (!vmuFound) {
+					dreamLinkVmus[i].push_back(vmu);
+				}
+			}
+
+			if (gameStart)
+			{
+				disablePhysicalVmuMemoryOption();
+			}
+			else if (gameEnd)
+			{
+				enablePhysicalVmuMemoryOption();
+			}
+		}
+		else if (i == 1 && ((dreamlink->getFunctionCode(i + 1) & MFID_8_Vibration) || (dev != nullptr && dev->get_device_type() == MDT_PurupuruPack)))
+		{
+			bool rumbleFound = false;
+			std::shared_ptr<DreamLinkPurupuru> rumble;
+			for (const std::shared_ptr<DreamLinkPurupuru>& purupuru : dreamLinkPurupurus)
+			{
+				if (purupuru->dreamlink.get() == dreamlink.get())
+				{
+					rumbleFound = true;
+					rumble = purupuru;
+					break;
+				}
+			}
+
+			if (gameStart || !rumbleFound)
+			{
+				if (!rumble)
+				{
+					rumble = std::make_shared<DreamLinkPurupuru>(dreamlink);
+				}
+				rumble->Setup(bus, i);
+
+				if (!rumbleFound) dreamLinkPurupurus.push_back(rumble);
+			}
+		}
+	}
+}
+
+void tearDownDreamLinkDevices(std::shared_ptr<DreamLink> dreamlink)
+{
+	const int bus = dreamlink->getBus();
+
+	for (int i = 0; i < 2; ++i)
+	{
+		for (std::list<std::shared_ptr<DreamLinkVmu>>::const_iterator iter = dreamLinkVmus[i].begin();
+			iter != dreamLinkVmus[i].end();)
+		{
+			if ((*iter)->dreamlink.get() == dreamlink.get())
+			{
+				DEBUG_LOG(MAPLE, "VMU teardown - Physical VMU: %s", (*iter)->useRealVmuMemory ? "true" : "false");
+				std::shared_ptr<maple_device> dev = maple_Create(MDT_SegaVMU);
+				dev->Setup(bus, 0);
+				if (!(*iter)->useRealVmuMemory)
+				{
+					(*iter)->copyOut(std::static_pointer_cast<maple_sega_vmu>(dev));
+				}
+				DEBUG_LOG(MAPLE, "VMU teardown - Copy completed");
+				iter = dreamLinkVmus[i].erase(iter);
+				break;
+			}
+			else
+			{
+				++iter;
+			}
+		}
+	}
+
+	std::size_t dreamLinkVmuCount = 0;
+	for (int i = 0; i < 2; ++i)
+	{
+		dreamLinkVmuCount += dreamLinkVmus[i].size();
+	}
+
+	if (dreamLinkVmuCount == 0)
+	{
+		enablePhysicalVmuMemoryOption();
+	}
+
+	for (std::list<std::shared_ptr<DreamLinkPurupuru>>::const_iterator iter = dreamLinkPurupurus.begin();
+		iter != dreamLinkPurupurus.end();)
+	{
+		if ((*iter)->dreamlink.get() == dreamlink.get())
+		{
+			std::shared_ptr<maple_device> dev = maple_Create(MDT_PurupuruPack);
+			dev->Setup(bus, 1);
+			iter = dreamLinkPurupurus.erase(iter);
+			break;
+		}
+		else
+		{
+			++iter;
+		}
+	}
+}
+
+#endif
