@@ -329,7 +329,7 @@ bool GamepadDevice::gamepad_axis_input(u32 code, int value)
 			int threshold = 16384;
 			if (code == leftTrigger || code == rightTrigger )
 				threshold = 100;
-				
+
 			if (std::abs(v) < threshold)
 				kcode[port] |=  key; // button released
 			else
@@ -447,24 +447,18 @@ bool GamepadDevice::find_mapping(int system /* = settings.platform.system */)
 	return false;
 }
 
-int GamepadDevice::GetGamepadCount()
-{
-	_gamepads_mutex.lock();
-	int count = _gamepads.size();
-	_gamepads_mutex.unlock();
-	return count;
+int GamepadDevice::GetGamepadCount() {
+	Lock _(_gamepads_mutex);
+	return _gamepads.size();
 }
 
 std::shared_ptr<GamepadDevice> GamepadDevice::GetGamepad(int index)
 {
-	_gamepads_mutex.lock();
-	std::shared_ptr<GamepadDevice> dev;
+	Lock _(_gamepads_mutex);
 	if (index >= 0 && index < (int)_gamepads.size())
-		dev = _gamepads[index];
+		return _gamepads[index];
 	else
-		dev = NULL;
-	_gamepads_mutex.unlock();
-	return dev;
+		return nullptr;
 }
 
 void GamepadDevice::save_mapping(int system /* = settings.platform.system */)
@@ -557,21 +551,22 @@ void GamepadDevice::Register(const std::shared_ptr<GamepadDevice>& gamepad)
 			setbuf(record_input, NULL);
 	}
 #endif
-	_gamepads_mutex.lock();
+	Lock _(_gamepads_mutex);
 	_gamepads.push_back(gamepad);
-	_gamepads_mutex.unlock();
 	MapleConfigMap::UpdateVibration = updateVibration;
+
+	gamepad->_is_registered = true;
+	gamepad->registered();
 }
 
 void GamepadDevice::Unregister(const std::shared_ptr<GamepadDevice>& gamepad)
 {
-	_gamepads_mutex.lock();
+	Lock _(_gamepads_mutex);
 	for (auto it = _gamepads.begin(); it != _gamepads.end(); it++)
 		if (*it == gamepad) {
 			_gamepads.erase(it);
 			break;
 		}
-	_gamepads_mutex.unlock();
 }
 
 void GamepadDevice::SaveMaplePorts()
@@ -584,6 +579,83 @@ void GamepadDevice::SaveMaplePorts()
 	}
 }
 
+s16 (&GamepadDevice::getTargetArray(DigAnalog axis))[4]
+{
+	switch (axis)
+	{
+	case DIGANA_LEFT:
+	case DIGANA_RIGHT:
+		return joyx;
+	case DIGANA_UP:;
+	case DIGANA_DOWN:
+		return joyy;
+	case DIGANA2_LEFT:
+	case DIGANA2_RIGHT:
+		return joyrx;
+	case DIGANA2_UP:
+	case DIGANA2_DOWN:
+		return joyry;
+	case DIGANA3_LEFT:
+	case DIGANA3_RIGHT:
+		return joy3x;
+	case DIGANA3_UP:
+	case DIGANA3_DOWN:
+		return joy3y;
+	default:
+		die("unknown axis");
+	}
+}
+
+void GamepadDevice::rampAnalog()
+{
+	Lock _(rampMutex);
+	if (lastAnalogUpdate == 0)
+		// also used as a flag that no analog ramping is needed on this device (yet)
+		return;
+
+	const u64 now = getTimeMs();
+	const int delta = std::round(static_cast<float>(now - lastAnalogUpdate) * AnalogRamp);
+	lastAnalogUpdate = now;
+	for (unsigned port = 0; port < std::size(digitalToAnalogState); port++)
+	{
+		for (int axis = 0; axis < 12; axis += 2)	// 3 sticks with 2 axes each
+		{
+			DigAnalog negDir = static_cast<DigAnalog>(1 << axis);
+			if ((rampAnalogState[port] & negDir) == 0)
+				// axis not active
+				continue;
+			DigAnalog posDir = static_cast<DigAnalog>(1 << (axis + 1));
+			const int socd = digitalToAnalogState[port] & (negDir | posDir);
+			s16& axisValue = getTargetArray(negDir)[port];
+			if (socd != 0 && socd != (negDir | posDir))
+			{
+				// One axis is pressed => ramp up
+				if (socd == posDir)
+					axisValue = std::min(32767, axisValue + delta);
+				else
+					axisValue = std::max(-32768, axisValue - delta);
+			}
+			else
+			{
+				// No axis is pressed (or both) => ramp down
+				if (axisValue > 0)
+					axisValue = std::max(0, axisValue - delta);
+				else if (axisValue < 0)
+					axisValue = std::min(0, axisValue + delta);
+				else
+					rampAnalogState[port] &= ~negDir;
+			}
+		}
+	}
+}
+
+void GamepadDevice::RampAnalog()
+{
+	Lock _(_gamepads_mutex);
+	for (auto& gamepad : _gamepads)
+		gamepad->rampAnalog();
+}
+
 #ifdef TEST_AUTOMATION
 #include "cfg/option.h"
 static bool replay_inited;
