@@ -22,7 +22,6 @@
 
 #include "rec_x86.h"
 #include "hw/sh4/sh4_core.h"
-#include "hw/sh4/sh4_interpreter.h"
 #include "hw/sh4/sh4_interrupts.h"
 #include "hw/sh4/sh4_mem.h"
 #include "hw/mem/addrspace.h"
@@ -70,11 +69,13 @@ void X86RegAlloc::Writeback_FPU(u32 reg, s8 nreg)
 
 struct DynaRBI : RuntimeBlockInfo
 {
-	DynaRBI(Sh4CodeBuffer *codeBuffer) : codeBuffer(codeBuffer) {}
+	DynaRBI(Sh4Context& sh4ctx, Sh4CodeBuffer& codeBuffer)
+	: sh4ctx(sh4ctx), codeBuffer(codeBuffer) {}
 	u32 Relink() override;
 
 private:
-	Sh4CodeBuffer *codeBuffer;
+	Sh4Context& sh4ctx;
+	Sh4CodeBuffer& codeBuffer;
 };
 
 
@@ -109,26 +110,26 @@ void X86Compiler::compile(RuntimeBlockInfo* block, bool force_checks, bool optim
 	if (mmu_enabled() && block->has_fpu_op)
 	{
 		Xbyak::Label fpu_enabled;
-		mov(eax, dword[&sr.status]);
+		mov(eax, dword[&sh4ctx.sr.status]);
 		test(eax, 0x8000);			// test SR.FD bit
 		jz(fpu_enabled);
 		push(Sh4Ex_FpuDisabled);	// exception code
 		push(block->vaddr);			// pc
 		call((void (*)())Do_Exception);
 		add(esp, 8);
-		mov(ecx, dword[&next_pc]);
+		mov(ecx, dword[&sh4ctx.pc]);
 		jmp((const void *)no_update);
 		L(fpu_enabled);
 	}
 
-	mov(eax, dword[&Sh4cntx.cycle_counter]);
+	mov(eax, dword[&sh4ctx.cycle_counter]);
 	test(eax, eax);
 	Xbyak::Label no_up;
 	jg(no_up);
 	mov(ecx, block->vaddr);
 	call((const void *)intc_sched);
 	L(no_up);
-	sub(dword[&Sh4cntx.cycle_counter], block->guest_cycles);
+	sub(dword[&sh4ctx.cycle_counter], block->guest_cycles);
 
 	regalloc.doAlloc(block);
 
@@ -181,7 +182,7 @@ u32 X86Compiler::relinkBlock(RuntimeBlockInfo* block)
 
 			mov(ecx, block->NextBlock);
 
-			cmp(dword[GetRegPtr(block->has_jcond ? reg_pc_dyn : reg_sr_T)], (u32)block->BlockType & 1);
+			cmp(dword[block->has_jcond ? &sh4ctx.jdyn : &sh4ctx.sr.T], (u32)block->BlockType & 1);
 			Xbyak::Label branch_not_taken;
 
 			jne(branch_not_taken, T_SHORT);
@@ -194,7 +195,7 @@ u32 X86Compiler::relinkBlock(RuntimeBlockInfo* block)
 	case BET_DynamicCall:
 	case BET_DynamicRet:
 		//next_pc = *jdyn;
-		mov(ecx, dword[GetRegPtr(reg_pc_dyn)]);
+		mov(ecx, dword[&sh4ctx.jdyn]);
 		break;
 
 	case BET_DynamicIntr:
@@ -202,16 +203,16 @@ u32 X86Compiler::relinkBlock(RuntimeBlockInfo* block)
 		if (block->BlockType == BET_DynamicIntr)
 		{
 			//next_pc = *jdyn;
-			mov(ecx, dword[GetRegPtr(reg_pc_dyn)]);
-			mov(dword[&next_pc], ecx);
+			mov(ecx, dword[&sh4ctx.jdyn]);
+			mov(dword[&sh4ctx.pc], ecx);
 		}
 		else
 		{
 			//next_pc = next_pc_value;
-			mov(dword[&next_pc], block->NextBlock);
+			mov(dword[&sh4ctx.pc], block->NextBlock);
 		}
 		call(UpdateINTC);
-		mov(ecx, dword[&next_pc]);
+		mov(ecx, dword[&sh4ctx.pc]);
 		break;
 
 	default:
@@ -226,7 +227,7 @@ u32 X86Compiler::relinkBlock(RuntimeBlockInfo* block)
 	{
 	case BET_Cond_0:
 	case BET_Cond_1:
-		cmp(dword[GetRegPtr(block->has_jcond ? reg_pc_dyn : reg_sr_T)], (u32)block->BlockType & 1);
+		cmp(dword[block->has_jcond ? &sh4ctx.jdyn : &sh4ctx.sr.T], (u32)block->BlockType & 1);
 
 		if (mmu_enabled())
 		{
@@ -268,7 +269,7 @@ u32 X86Compiler::relinkBlock(RuntimeBlockInfo* block)
 	case BET_DynamicRet:
 	case BET_DynamicCall:
 	case BET_DynamicJump:
-		mov(ecx, dword[GetRegPtr(reg_pc_dyn)]);
+		mov(ecx, dword[&sh4ctx.jdyn]);
 		jmp((const void *)no_update);
 
 		break;
@@ -298,16 +299,16 @@ u32 X86Compiler::relinkBlock(RuntimeBlockInfo* block)
 	case BET_DynamicIntr:
 		if (block->BlockType == BET_StaticIntr)
 		{
-			mov(dword[&next_pc], block->NextBlock);
+			mov(dword[&sh4ctx.pc], block->NextBlock);
 		}
 		else
 		{
-			mov(eax, dword[GetRegPtr(reg_pc_dyn)]);
-			mov(dword[&next_pc], eax);
+			mov(eax, dword[&sh4ctx.jdyn]);
+			mov(dword[&sh4ctx.pc], eax);
 		}
 		call(UpdateINTC);
 
-		mov(ecx, dword[&next_pc]);
+		mov(ecx, dword[&sh4ctx.pc]);
 		jmp((const void *)no_update);
 
 		break;
@@ -324,7 +325,7 @@ u32 X86Compiler::relinkBlock(RuntimeBlockInfo* block)
 
 u32 DynaRBI::Relink()
 {
-	X86Compiler *compiler = new X86Compiler(*codeBuffer, (u8*)code + relink_offset);
+	X86Compiler *compiler = new X86Compiler(sh4ctx, codeBuffer, (u8*)code + relink_offset);
 	u32 codeSize = compiler->relinkBlock(this);
 	delete compiler;
 
@@ -359,7 +360,13 @@ void X86Compiler::ngen_CC_param(const shil_opcode& op, const shil_param& param,
 		//push the ptr itself
 		case CPT_ptr:
 			verify(param.is_reg());
-			push((uintptr_t)param.reg_ptr());
+			push((uintptr_t)param.reg_ptr(sh4ctx));
+			CC_stackSize += 4;
+			unwinder.allocStackPtr(getCurr(), 4);
+			break;
+
+		case CPT_sh4ctx:
+			push((uintptr_t)&sh4ctx);
 			CC_stackSize += 4;
 			unwinder.allocStackPtr(getCurr(), 4);
 			break;
@@ -377,8 +384,8 @@ void X86Compiler::ngen_CC_param(const shil_opcode& op, const shil_param& param,
 
 		// store from ST(0)
 		case CPT_f32rv:
-			fstp(dword[param.reg_ptr()]);
-			movss(regalloc.MapXRegister(param), dword[param.reg_ptr()]);
+			fstp(dword[param.reg_ptr(sh4ctx)]);
+			movss(regalloc.MapXRegister(param), dword[param.reg_ptr(sh4ctx)]);
 			break;
 	}
 }
@@ -440,33 +447,33 @@ void X86Compiler::genMainloop()
 	Xbyak::Label longjmpLabel;
 	L(longjmpLabel);
 
-	mov(ecx, dword[&Sh4cntx.pc]);
+	mov(ecx, dword[&sh4ctx.pc]);
 
 	//next_pc _MUST_ be on ecx
 	Xbyak::Label cleanup;
 //no_update:
 	Xbyak::Label no_updateLabel;
 	L(no_updateLabel);
-	mov(edx, dword[&Sh4cntx.CpuRunning]);
+	mov(edx, dword[&sh4ctx.CpuRunning]);
 	cmp(edx, 0);
 	jz(cleanup);
 	if (!mmu_enabled())
 	{
 		mov(esi, ecx);	// save sh4 pc in ESI, used below if FPCB is still empty for this address
-		mov(eax, (size_t)&p_sh4rcb->fpcb[0]);
+		mov(eax, (uintptr_t)&sh4ctx + sizeof(Sh4Context) - sizeof(Sh4RCB) + offsetof(Sh4RCB, fpcb));	// address of fpcb[0]
 		and_(ecx, RAM_SIZE_MAX - 2);
 		jmp(dword[eax + ecx * 2]);
 	}
 	else
 	{
-		mov(dword[&next_pc], ecx);
+		mov(dword[&sh4ctx.pc], ecx);
 		call((void *)bm_GetCodeByVAddr);
 		jmp(eax);
 	}
 
 //cleanup:
 	L(cleanup);
-	mov(dword[&next_pc], ecx);
+	mov(dword[&sh4ctx.pc], ecx);
 #ifndef _WIN32
 	// 16-byte alignment
 	add(esp, 12);
@@ -482,7 +489,7 @@ void X86Compiler::genMainloop()
 	Xbyak::Label do_iter;
 	L(do_iter);
 	add(esp, 4);	// pop intc_sched() return address
-	mov(ecx, dword[&Sh4cntx.pc]);
+	mov(ecx, dword[sh4ctx.pc]);
 	jmp(no_updateLabel);
 
 //ngen_LinkBlock_Shared_stub:
@@ -504,8 +511,8 @@ void X86Compiler::genMainloop()
 	unwinder.endProlog(0);
 	Xbyak::Label intc_schedLabel;
 	L(intc_schedLabel);
-	add(dword[&Sh4cntx.cycle_counter], SH4_TIMESLICE);
-	mov(dword[&Sh4cntx.pc], ecx);
+	add(dword[&sh4ctx.cycle_counter], SH4_TIMESLICE);
+	mov(dword[&sh4ctx.pc], ecx);
 	call((void *)UpdateSystem_INTC);
 	cmp(eax, 0);
 	jnz(do_iter);
@@ -526,7 +533,7 @@ void X86Compiler::genMainloop()
 //ngen_LinkBlock_Generic_stub:
 	Xbyak::Label ngen_LinkBlock_Generic_label;
 	L(ngen_LinkBlock_Generic_label);
-	mov(edx, dword[&Sh4cntx.jdyn]);
+	mov(edx, dword[&sh4ctx.jdyn]);
 	jmp(ngen_LinkBlock_Shared_stub);
 
 	genMemHandlers();
@@ -569,7 +576,7 @@ void X86Compiler::genMainloop()
 		Xbyak::Label jumpblockLabel;
 		cmp(eax, 0);
 		jne(jumpblockLabel);
-		mov(ecx, dword[&next_pc]);
+		mov(ecx, dword[&sh4ctx.pc]);
 		jmp(no_updateLabel);
 		L(jumpblockLabel);
 	}
@@ -620,7 +627,7 @@ bool X86Compiler::genReadMemImmediate(const shil_opcode& op, RuntimeBlockInfo* b
 			else
 			{
 				movsx(eax, byte[ptr]);
-				mov(dword[op.rd.reg_ptr()], eax);
+				mov(dword[op.rd.reg_ptr(sh4ctx)], eax);
 			}
 			break;
 
@@ -630,7 +637,7 @@ bool X86Compiler::genReadMemImmediate(const shil_opcode& op, RuntimeBlockInfo* b
 			else
 			{
 				movsx(eax, word[ptr]);
-				mov(dword[op.rd.reg_ptr()], eax);
+				mov(dword[op.rd.reg_ptr(sh4ctx)], eax);
 			}
 			break;
 
@@ -642,7 +649,7 @@ bool X86Compiler::genReadMemImmediate(const shil_opcode& op, RuntimeBlockInfo* b
 			else
 			{
 				mov(eax, dword[ptr]);
-				mov(dword[op.rd.reg_ptr()], eax);
+				mov(dword[op.rd.reg_ptr(sh4ctx)], eax);
 			}
 			break;
 
@@ -655,7 +662,7 @@ bool X86Compiler::genReadMemImmediate(const shil_opcode& op, RuntimeBlockInfo* b
 			else
 			{
 				movq(xmm0, qword[ptr]);
-				movq(qword[op.rd.reg_ptr()], xmm0);
+				movq(qword[op.rd.reg_ptr(sh4ctx)], xmm0);
 			}
 			break;
 
@@ -674,11 +681,11 @@ bool X86Compiler::genReadMemImmediate(const shil_opcode& op, RuntimeBlockInfo* b
 			// Need to call the handler twice
 			mov(ecx, addr);
 			genCall((void (DYNACALL *)())ptr);
-			mov(dword[op.rd.reg_ptr()], eax);
+			mov(dword[op.rd.reg_ptr(sh4ctx)], eax);
 
 			mov(ecx, addr + 4);
 			genCall((void (DYNACALL *)())ptr);
-			mov(dword[op.rd.reg_ptr() + 1], eax);
+			mov(dword[op.rd.reg_ptr(sh4ctx) + 1], eax);
 		}
 		else
 		{
@@ -742,7 +749,7 @@ bool X86Compiler::genWriteMemImmediate(const shil_opcode& op, RuntimeBlockInfo*
 				mov(byte[ptr], (u8)op.rs2.imm_value());
 			else
 			{
-				mov(al, byte[op.rs2.reg_ptr()]);
+				mov(al, byte[op.rs2.reg_ptr(sh4ctx)]);
 				mov(byte[ptr], al);
 			}
 			break;
@@ -754,7 +761,7 @@ bool X86Compiler::genWriteMemImmediate(const shil_opcode& op, RuntimeBlockInfo*
 				mov(word[ptr], (u16)op.rs2.imm_value());
 			else
 			{
-				mov(cx, word[op.rs2.reg_ptr()]);
+				mov(cx, word[op.rs2.reg_ptr(sh4ctx)]);
 				mov(word[ptr], cx);
 			}
 			break;
@@ -768,7 +775,7 @@ bool X86Compiler::genWriteMemImmediate(const shil_opcode& op, RuntimeBlockInfo*
 				mov(dword[ptr], op.rs2.imm_value());
 			else
 			{
-				mov(ecx, dword[op.rs2.reg_ptr()]);
+				mov(ecx, dword[op.rs2.reg_ptr(sh4ctx)]);
 				mov(dword[ptr], ecx);
 			}
 			break;
@@ -781,7 +788,7 @@ bool X86Compiler::genWriteMemImmediate(const shil_opcode& op, RuntimeBlockInfo*
 			}
 			else
 			{
-				movq(xmm0, qword[op.rs2.reg_ptr()]);
+				movq(xmm0, qword[op.rs2.reg_ptr(sh4ctx)]);
 				movq(qword[ptr], xmm0);
 			}
 			break;
@@ -810,7 +817,7 @@ void X86Compiler::checkBlock(bool smc_checks, RuntimeBlockInfo* block)
 
 	if (mmu_enabled())
 	{
-		mov(eax, dword[&next_pc]);
+		mov(eax, dword[&sh4ctx.pc]);
 		cmp(eax, block->vaddr);
 		jne(reinterpret_cast<const void*>(ngen_blockcheckfail));
 	}
@@ -843,8 +850,9 @@ public:
 		sh4Dynarec = this;
 	}
 
-	void init(Sh4CodeBuffer& codeBuffer) override
+	void init(Sh4Context& sh4ctx, Sh4CodeBuffer& codeBuffer) override
 	{
+		this->sh4ctx = &sh4ctx;
 		this->codeBuffer = &codeBuffer;
 	}
 
@@ -858,7 +866,7 @@ public:
 		if (::mainloop != nullptr)
 			return;
 
-		compiler = new X86Compiler(*codeBuffer);
+		compiler = new X86Compiler(*sh4ctx, *codeBuffer);
 
 		try {
 			compiler->genMainloop();
@@ -877,10 +885,10 @@ public:
 		::mainloop = nullptr;
 		unwinder.clear();
 
-		if (p_sh4rcb->cntx.CpuRunning)
+		if (sh4ctx->CpuRunning)
 		{
 			// Force the dynarec out of mainloop() to regenerate it
-			p_sh4rcb->cntx.CpuRunning = 0;
+			sh4ctx->CpuRunning = 0;
 			restarting = true;
 		}
 		else
@@ -890,7 +898,7 @@ public:
 	RuntimeBlockInfo* allocateBlock() override
 	{
 		generate_mainloop();
-		return new DynaRBI(codeBuffer);
+		return new DynaRBI(*sh4ctx, *codeBuffer);
 	}
 
 	void mainloop(void* v_cntx) override
@@ -912,7 +920,7 @@ public:
 
 	void compile(RuntimeBlockInfo* block, bool smc_checks, bool optimise) override
 	{
-		compiler = new X86Compiler(*codeBuffer);
+		compiler = new X86Compiler(*sh4ctx, *codeBuffer);
 
 		try {
 			compiler->compile(block, smc_checks, optimise);
@@ -929,7 +937,7 @@ public:
 			// init() not called yet
 			return false;
 		u8 *rewriteAddr = *(u8 **)context.esp - 5;
-		X86Compiler *compiler = new X86Compiler(*codeBuffer, rewriteAddr);
+		X86Compiler *compiler = new X86Compiler(*sh4ctx, *codeBuffer, rewriteAddr);
 		bool rv = compiler->rewriteMemAccess(context);
 		delete compiler;
 
@@ -957,6 +965,7 @@ public:
 	}
 
 private:
+	Sh4Context *sh4ctx = nullptr;
 	Sh4CodeBuffer *codeBuffer = nullptr;
 	X86Compiler *compiler = nullptr;
 	bool restarting = false;
