@@ -17,40 +17,34 @@
 	 along with reicast.  If not, see <https://www.gnu.org/licenses/>.
  */
 #pragma once
-
-#include "TexCache.h"
-#include "stdclass.h"
-
+#include "texconv.h"
 #include <string>
-#include <vector>
 #include <map>
-#include <mutex>
+#include <memory>
 
-class CustomTexture {
+class BaseTextureCacheData;
+class WorkerThread;
+
+class CustomTexture
+{
 public:
-	CustomTexture() : loader_thread(loader_thread_func, this, "CustomTexLoader") {}
-	~CustomTexture() { Terminate(); }
-	u8* LoadCustomTexture(u32 hash, int& width, int& height);
+	~CustomTexture();
 	void LoadCustomTextureAsync(BaseTextureCacheData *texture_data);
 	void DumpTexture(u32 hash, int w, int h, TextureType textype, void *src_buffer);
 	void Terminate();
 
 private:
-	bool Init();
-	void LoaderThread();
-	std::string GetGameId();
-	void LoadMap();
-	
-	static void *loader_thread_func(void *param) { ((CustomTexture *)param)->LoaderThread(); return NULL; }
+	bool init();
+	u8* loadTexture(u32 hash, int& width, int& height);
+	void loadTexture(BaseTextureCacheData *texture);
+	std::string getGameId();
+	void loadMap();
 	
 	bool initialized = false;
 	bool custom_textures_available = false;
 	std::string textures_path;
-	cThread loader_thread;
-	cResetEvent wakeup_thread;
-	std::vector<BaseTextureCacheData *> work_queue;
-	std::mutex work_queue_mutex;
 	std::map<u32, std::string> texture_map;
+	std::unique_ptr<WorkerThread> loaderThread;
 };
 
 extern CustomTexture custom_texture;
