@@ -21,7 +21,6 @@
 #include "hw/pvr/ta.h"
 #include "hw/pvr/pvr_mem.h"
 #include "ui/gui.h"
-#include "rend/tileclip.h"
 #include "rend/sorter.h"
 
 #include <memory>
@@ -159,7 +158,7 @@ bool DX11Renderer::Init()
 	quad->init(device, deviceContext, shaders);
 	n2Helper.init(device, deviceContext);
 
-	fog_needs_update = true;
+	updateFogTable = true;
 
 	if (!success)
 	{
@@ -319,8 +318,10 @@ BaseTextureCacheData *DX11Renderer::GetTexture(TSP tsp, TCW tcw)
 
 void DX11Renderer::Process(TA_context* ctx)
 {
-	if (KillTex)
+	if (resetTextureCache) {
 		texCache.Clear();
+		resetTextureCache = false;
+	}
 	texCache.Cleanup();
 
 	ta_parse(ctx, true);
@@ -514,7 +515,7 @@ bool DX11Renderer::Render()
 #ifndef LIBRETRO
 		deviceContext->OMSetRenderTargets(1, &theDX11Context.getRenderTarget().get(), nullptr);
 		displayFramebuffer();
-		DrawOSD(false);
+		drawOSD();
 		renderVideoRouting();
 		theDX11Context.setFrameRendered();
 #else
@@ -524,6 +525,7 @@ bool DX11Renderer::Render()
 #endif
 		frameRendered = true;
 		frameRenderedOnce = true;
+		clearLastFrame = false;
 	}
 
 	return !is_rtt;
@@ -605,6 +607,19 @@ void DX11Renderer::setCullMode(int mode)
 	deviceContext->RSSetState(rasterizer);
 }
 
+TileClipping DX11Renderer::setTileClip(u32 tileclip, int clip_rect[4])
+{
+	TileClipping clipmode = GetTileClip(tileclip, matrices.GetViewportMatrix(), clip_rect);
+	if (clipmode == TileClipping::Outside) {
+		RECT rect { clip_rect[0], clip_rect[1], clip_rect[0] + clip_rect[2], clip_rect[1] + clip_rect[3] };
+		deviceContext->RSSetScissorRects(1, &rect);
+	}
+	else {
+		deviceContext->RSSetScissorRects(1, &scissorRect);
+	}
+	return clipmode;
+}
+
 template <u32 Type, bool SortingEnabled>
 void DX11Renderer::setRenderState(const PolyParam *gp)
 {
@@ -623,7 +638,7 @@ void DX11Renderer::setRenderState(const PolyParam *gp)
 	int fog_ctrl = config::Fog ? gp->tsp.FogCtrl : 2;
 
 	int clip_rect[4] = {};
-	TileClipping clipmode = GetTileClip(gp->tileclip, matrices.GetViewportMatrix(), clip_rect);
+	TileClipping clipmode = setTileClip(gp->tileclip, clip_rect);
 	DX11Texture *texture = (DX11Texture *)gp->texture;
 	int gpuPalette = texture == nullptr || !texture->gpuPalette ? 0
 			: gp->tsp.FilterMode + 1;
@@ -662,21 +677,12 @@ void DX11Renderer::setRenderState(const PolyParam *gp)
 			constants.paletteIndex = (float)((gp->tcw.PalSelect >> 4) << 8);
 	}
 
-	if (clipmode == TileClipping::Outside)
+	if (clipmode == TileClipping::Inside)
 	{
-		RECT rect { clip_rect[0], clip_rect[1], clip_rect[0] + clip_rect[2], clip_rect[1] + clip_rect[3] };
-		deviceContext->RSSetScissorRects(1, &rect);
-	}
-	else
-	{
-		deviceContext->RSSetScissorRects(1, &scissorRect);
-		if (clipmode == TileClipping::Inside)
-		{
-			constants.clipTest[0] = (float)clip_rect[0];
-			constants.clipTest[1] = (float)clip_rect[1];
-			constants.clipTest[2] = (float)(clip_rect[0] + clip_rect[2]);
-			constants.clipTest[3] = (float)(clip_rect[1] + clip_rect[3]);
-		}
+		constants.clipTest[0] = (float)clip_rect[0];
+		constants.clipTest[1] = (float)clip_rect[1];
+		constants.clipTest[2] = (float)(clip_rect[0] + clip_rect[2]);
+		constants.clipTest[3] = (float)(clip_rect[1] + clip_rect[3]);
 	}
 	if (constants.trilinearAlpha != 1.f || gpuPalette != 0 || clipmode == TileClipping::Inside)
 	{
@@ -827,7 +833,6 @@ void DX11Renderer::drawModVols(int first, int count)
 
 	deviceContext->PSSetShader(shaders->getModVolShader(), nullptr, 0);
 
-	deviceContext->RSSetScissorRects(1, &scissorRect);
 	setCullMode(0);
 
 	const ModifierVolumeParam *params = &pvrrc.global_param_mvo[first];
@@ -859,6 +864,10 @@ void DX11Renderer::drawModVols(int first, int count)
 			// XOR'ing (closed volume)
 			deviceContext->OMSetDepthStencilState(depthStencilStates.getMVState(DepthStencilStates::Xor), 0);
 
+		int clip_rect[4] = {};
+		setTileClip(param.tileclip, clip_rect);
+		// TODO inside clipping
+
 		if (param.count > 0)
 		{
 			setCullMode(param.isp.CullMode);
@@ -877,6 +886,7 @@ void DX11Renderer::drawModVols(int first, int count)
 	setCullMode(0);
 	//enable color writes
 	deviceContext->OMSetBlendState(blendStates.getState(true, 4, 5), nullptr, 0xffffffff);
+	deviceContext->RSSetScissorRects(1, &scissorRect);
 
 	//black out any stencil with '1'
 	//only pixels that are Modvol enabled, and in area 1
@@ -930,7 +940,7 @@ void DX11Renderer::drawStrips()
 
 bool DX11Renderer::RenderLastFrame()
 {
-	if (!frameRenderedOnce)
+	if (!frameRenderedOnce || clearLastFrame)
 		return false;
 	displayFramebuffer();
 	return true;
@@ -1022,7 +1032,7 @@ void DX11Renderer::RenderFramebuffer(const FramebufferInfo& info)
 
 	deviceContext->OMSetRenderTargets(1, &theDX11Context.getRenderTarget().get(), nullptr);
 	displayFramebuffer();
-	DrawOSD(false);
+	drawOSD();
 	renderVideoRouting();
 	theDX11Context.setFrameRendered();
 #else
@@ -1032,6 +1042,7 @@ void DX11Renderer::RenderFramebuffer(const FramebufferInfo& info)
 #endif
 	frameRendered = true;
 	frameRenderedOnce = true;
+	clearLastFrame = false;
 }
 
 void DX11Renderer::setBaseScissor()
@@ -1216,9 +1227,9 @@ void DX11Renderer::readRttRenderTarget(u32 texAddress)
 
 void DX11Renderer::updatePaletteTexture()
 {
-	if (palette_updated)
+	if (updatePalette)
 	{
-		palette_updated = false;
+		updatePalette = false;
 		deviceContext->UpdateSubresource(paletteTexture, 0, nullptr, palette32_ram, 32 * sizeof(u32), 32 * sizeof(u32) * 32);
 	}
     deviceContext->PSSetShaderResources(1, 1, &paletteTextureView.get());
@@ -1229,9 +1240,9 @@ void DX11Renderer::updateFogTexture()
 {
 	if (!config::Fog)
 		return;
-	if (fog_needs_update)
+	if (updateFogTable)
 	{
-		fog_needs_update = false;
+		updateFogTable = false;
 		u8 temp_tex_buffer[256];
 		MakeFogTexture(temp_tex_buffer);
 
@@ -1241,10 +1252,10 @@ void DX11Renderer::updateFogTexture()
     deviceContext->PSSetSamplers(2, 1, &samplers->getSampler(true).get());
 }
 
-void DX11Renderer::DrawOSD(bool clear_screen)
+void DX11Renderer::drawOSD()
 {
 #ifndef LIBRETRO
-	theDX11Context.setOverlay(!clear_screen);
+	theDX11Context.setOverlay(true);
 	gui_display_osd();
 	theDX11Context.setOverlay(false);
 #endif
