@@ -29,6 +29,7 @@
 #import <dlfcn.h>
 
 #import "PadViewController.h"
+#import "EditPadViewController.h"
 #import "EmulatorView.h"
 
 #include "types.h"
@@ -43,7 +44,6 @@
 #include "ios_mouse.h"
 #include "oslib/oslib.h"
 
-//@import AltKit;
 #import "AltKit-Swift.h"
 
 static std::string iosJitStatus;
@@ -92,6 +92,7 @@ static void updateAudioSession(Event event, void *)
 			switch (config::MapleMainDevices[bus])
 			{
 				case MDT_SegaController:
+				case MDT_SegaControllerXL:
 					for (int port = 0; port < 2; port++)
 						if (config::MapleExpansionDevices[bus][port] == MDT_Microphone)
 							hasMicrophone = true;
@@ -99,6 +100,7 @@ static void updateAudioSession(Event event, void *)
 				case MDT_LightGun:
 				case MDT_AsciiStick:
 				case MDT_TwinStick:
+				case MDT_RacingController:
 					if (config::MapleExpansionDevices[bus][0] == MDT_Microphone)
 						hasMicrophone = true;
 					break;
@@ -147,6 +149,7 @@ static void updateAudioSession(Event event, void *)
 
 @property (strong, nonatomic) EAGLContext *context;
 @property (strong, nonatomic) PadViewController *padController;
+@property (strong, nonatomic) EditPadViewController *editPadController;
 
 @property (nonatomic) iCadeReaderView* iCadeReader;
 @property (nonatomic, strong) id gamePadConnectObserver;
@@ -216,6 +219,7 @@ static void updateAudioSession(Event event, void *)
 
 #if !TARGET_OS_TV
 	self.padController = [[PadViewController alloc] initWithNibName:@"PadViewController" bundle:nil];
+	self.editPadController = [[EditPadViewController alloc] initWithNibName:@"EditPadViewController" bundle:nil];
 #endif
 	
     self.context = [[EAGLContext alloc] initWithAPI:kEAGLRenderingAPIOpenGLES3];
@@ -294,6 +298,12 @@ static void updateAudioSession(Event event, void *)
 	self.padController.view.autoresizingMask = UIViewAutoresizingFlexibleWidth | UIViewAutoresizingFlexibleHeight;
 	if (IOSGamepad::controllerConnected())
 		[self.padController hideController];
+
+	[self addChildViewController:self.editPadController];
+	self.editPadController.view.frame = self.view.bounds;
+	self.editPadController.view.translatesAutoresizingMaskIntoConstraints = YES;
+	self.editPadController.view.autoresizingMask = UIViewAutoresizingFlexibleWidth | UIViewAutoresizingFlexibleHeight;
+	[self.editPadController hideController];
 #endif
 
     self.iCadeReader = [[iCadeReaderView alloc] init];
@@ -313,6 +323,13 @@ static void updateAudioSession(Event event, void *)
 	}
 	settings.display.dpi = 160.f * scale;
 	initRenderApi();
+#if TARGET_OS_SIMULATOR == 1 && HOST_CPU == CPU_ARM64
+	if (config::RendererType.get() == RenderType::OpenGL)
+	{
+		NSLog(@"🚨🚨🚨 OpenGL renderer is not supported in Apple Silicon Mac 🚨🚨🚨\n");
+		raise(SIGTRAP);
+	}
+#endif
 	mainui_init();
 
 	[self altKitStart];
@@ -690,6 +707,20 @@ bool checkTryDebug()
 #endif
 	mainui_rend_frame();
 }
+
+- (void)setVGamepadEditMode:(BOOL)editing
+{
+#if !TARGET_OS_TV
+	if (editing != [self.editPadController isControllerVisible])
+	{
+		if (editing)
+			[self.editPadController showController:self.view];
+		else
+			[self.editPadController hideController];
+	}
+#endif
+}
+
 /*
 - (void)pickIosFile
 {
@@ -744,3 +775,12 @@ const char *getIosJitStatus()
 	}
 	return iosJitStatus.c_str();
 }
+
+namespace vgamepad
+{
+
+void setEditMode(bool editing) {
+	[flycastViewController setVGamepadEditMode:editing];
+}
+
+}
