@@ -23,14 +23,6 @@
 #include "types.h"
 #include "hw/sh4/sh4_if.h"
 #include "hw/mem/addrspace.h"
-#include "hw/sh4/sh4_core.h"
-#undef r
-#undef fr
-#undef sr
-#undef mac
-#undef gbr
-#undef fpscr
-#undef fpul
 
 constexpr u32 REG_MAGIC = 0xbaadf00d;
 
@@ -43,17 +35,18 @@ protected:
 	virtual void PrepareOp(u16 op, u16 op2 = 0, u16 op3 = 0) = 0;
 	virtual void RunOp(int numOp = 1) = 0;
 
-	void ClearRegs() {
-		for (int i = 0; i < 16; i++)
-			r(i) = REG_MAGIC;
-		for (int i = 0; i < 32; i++)
-			*(u32 *)&ctx->xffr[i] = REG_MAGIC;
-		sh4_sr_SetFull(0x700000F0);
+	void ClearRegs()
+	{
+		std::fill(std::begin(ctx->r), std::end(ctx->r), REG_MAGIC);
+		std::fill(std::begin(reinterpret_cast<u32 (&)[16]>(ctx->xf)), std::end(reinterpret_cast<u32 (&)[16]>(ctx->xf)), REG_MAGIC);
+		std::fill(std::begin(reinterpret_cast<u32 (&)[16]>(ctx->fr)), std::end(reinterpret_cast<u32 (&)[16]>(ctx->fr)), REG_MAGIC);
+		sr().setFull(0x700000F0);
 		mac() = 0;
 		gbr() = REG_MAGIC;
 		checkedRegs.clear();
 	}
-	void AssertState() {
+	void AssertState()
+	{
 		for (int i = 0; i < 16; i++)
 			if (checkedRegs.count(&ctx->r[i]) == 0)
 				ASSERT_CLEAN_REG(i);
@@ -61,10 +54,15 @@ protected:
 		{
 			ASSERT_EQ(ctx->gbr, REG_MAGIC);
 		}
-		for (int i = 0; i < 32; i++)
-			if (checkedRegs.count((u32 *)&ctx->xffr[i]) == 0)
+		for (int i = 0; i < 16; i++)
+			if (checkedRegs.count((u32 *)&ctx->xf[i]) == 0)
 			{
-				ASSERT_EQ(*(u32 *)&ctx->xffr[i], REG_MAGIC);
+				ASSERT_EQ(*(u32 *)&ctx->xf[i], REG_MAGIC);
+			}
+		for (int i = 0; i < 16; i++)
+			if (checkedRegs.count((u32 *)&ctx->fr[i]) == 0)
+			{
+				ASSERT_EQ(*(u32 *)&ctx->fr[i], REG_MAGIC);
 			}
 	}
 	static u16 Rm(int r) { return r << 4; }
@@ -78,20 +76,20 @@ protected:
 	u32& mach() { return ctx->mac.l; }
 	u32& macl() { return ctx->mac.h; }
 	sr_t& sr() { return ctx->sr; }
-	f32& fr(int regNum) { checkedRegs.insert((u32 *)&ctx->xffr[regNum + 16]); return ctx->xffr[regNum + 16]; }
+	f32& fr(int regNum) { checkedRegs.insert((u32 *)&ctx->fr[regNum]); return ctx->fr[regNum]; }
 	double getDr(int regNum) {
-		checkedRegs.insert((u32 *)&ctx->xffr[regNum * 2 + 16]);
-		checkedRegs.insert((u32 *)&ctx->xffr[regNum * 2 + 1 + 16]);
-		return GetDR(regNum);
+		checkedRegs.insert((u32 *)&ctx->fr[regNum * 2]);
+		checkedRegs.insert((u32 *)&ctx->fr[regNum * 2 + 1]);
+		return ctx->getDR(regNum);
 	}
 	void setDr(int regNum, double d) {
-		checkedRegs.insert((u32 *)&ctx->xffr[regNum * 2 + 16]);
-		checkedRegs.insert((u32 *)&ctx->xffr[regNum * 2 + 1 + 16]);
-		SetDR(regNum, d);
+		checkedRegs.insert((u32 *)&ctx->fr[regNum * 2]);
+		checkedRegs.insert((u32 *)&ctx->fr[regNum * 2 + 1]);
+		ctx->setDR(regNum, d);
 	}
 
 	Sh4Context *ctx;
-	sh4_if sh4;
+	Sh4Executor *sh4;
 	std::set<u32 *> checkedRegs;
 	static constexpr u32 START_PC = 0xAC000000;
 
